@@ -1,12 +1,9 @@
 #!/usr/bin/python
-#
-# $Id: pytwitter.py 175 2009-05-10 00:29:23Z nsheridan@EVIL.IE $
-#
 
 """ Twitter api class.
 See http://apiwiki.twitter.com/REST+API+Documentation for api docs
 
-In short, converts 'method_action' to 'http://twitter.com/method/action.xml'
+In short, converts 'method_action' to 'http://twitter.com/method/action.json'
 using the magic of __getattr__. Believe it!
 
 Basic usage:
@@ -20,7 +17,7 @@ twitter.friendships_create(id="biz")
 ... and so on.
 
 Unless an exception is raised, full output is returned in the requested format
-Default: xml. Optional: json.
+Default: json.
 """
 
 import re
@@ -48,45 +45,61 @@ class pytwitter:
   username:: twitter username
   password:: twitter account password
   url (optional):: alternate api url (ex. http://identi.ca/api)
-  format:: xml or json (default: xml)
+
+  Methods are converted to corresponding api urls so that:
+    pytwitter.statuses_update(status='abcdef', format='json')
+  becomes:
+    http://twitter.com/statuses/update.json?status=abcdef
   """
-  def __init__(
-      self, username=None, password=None,
-      url='http://twitter.com', format='xml'):
+  def __init__(self, username=None, password=None, url='http://twitter.com'):
     self.url = url
     self.username = username
     self.password = password
-    self.format = format
 
   def __getattr__(self, method):
     def method(_self=self, _method=method, **params):
       """ Dynamic api method constructor
       Takes: A valid twitter method
-      Returns: xml or json output
+      Returns: Output from the server in the requested format (e.g. json)
       """
-      # Exception case for 'direct_messages'
+      # Some methods are POST only. Default to sending a GET.
+      use_post = False
+      post_only_methods = ('statuses_update', 'statuses_delete',
+          'account_end_session', 'friendships_destroy', 'friendships_create',
+          'direct_messages_destroy', 'direct_messages_new',
+          'account_update_delivery_service', 'account_update_profile_colors',
+          'account_update_profile_image', 'favorites_create',
+          'account_update_profile_background_image', 'account_update_profile',
+          'notifications_follow', 'notifications_leave', 'blocks_create',
+          'blocks_destroy', 'favorites_destroy', 'oauth_access_token',
+          'saved_searches_destroy', 'saved_searches_create')
+      if _method in post_only_methods:
+        use_post = True
+      # Exception case for 'direct_messages' and 'saved_searches'
       # Can't just replace('_', '/') for this one
       if 'direct_messages' in _method:
         # _method should look like one of:
         #  direct_messages.format
         #  direct_messages/method.format
         _method = re.sub(r'^(direct_messages)_(.+)$', r'\1/\2', _method)
+      elif 'saved_searches' in _method:
+        #  saved_searches.format
+        #  saved_searches/method.format
+        _method = re.sub(r'^(saved_searches)_(.+)$', r'\1/\2', _method)
       else:
         # Everything else.
         _method = _method.replace('_', '/', 1)
-      # Some methods are POST only. Default to sending a GET.
-      use_post = False
-      post_only_methods = ['statuses/update', 'statuses/delete',
-          'account/endsession', 'friendships/destroy', 'friendships/create',
-          'direct_messages/destroy', 'direct_messages/new',
-          'account/update_delivery_service', 'account/update_profile_colors',
-          'account/update_profile_image', 'favorites/create',
-          'account/update_profile_background_image', 'account/update_profile',
-          'notifications/follow', 'notifications/leave', 'blocks/create',
-          'blocks/destroy', 'favorites/destroy']
-      if _method in post_only_methods:
-        use_post = True
-      url = '%s/%s.%s' % (_self.url, _method, _self.format)
+      # Set the format. Fallback to json.
+      # format='' implies no format requested (for e.g. oauth methods)
+      if params.has_key('format'):
+        if params['format'] == '':
+          format = ''
+        else:
+          format = '.%s' % params['format']
+        del params['format']
+      else:
+        format = '.json'
+      url = '%s/%s%s' % (_self.url, _method, format)
       for key in params.keys():
         params[key] = str(params[key])
       params = urllib.urlencode(params)
@@ -96,7 +109,7 @@ class pytwitter:
 
   def _send_data(self, url, data, use_post=False):
     """ Authenticates with the server
-    Sends the request and returns the xml or json response
+    Sends the request and returns a response in the requested format (e.g json)
     Raises TwitterError exception on urllib2.UrlError
     """
     try:
@@ -122,6 +135,10 @@ def main():
   """ Test client """
   twitter = pytwitter()
   print twitter.help_test()
+  search_client = pytwitter(url='http://search.twitter.com')
+  print search_client.search(q='twitter', format='rss')
+  # This one fails with a 401 but should at least get the url correct
+  print twitter.oauth_request_token(format='')
 
 if __name__ == '__main__':
   main()
